package net.wanji.opt.controller;

import cn.hutool.core.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.bo.CrossIdAndStartEndDateBO;
import net.wanji.databus.bo.HeatMapBO;
import net.wanji.opt.bo.AbnormalDetailBO;
import net.wanji.opt.bo.MetricsTurnAndLaneBO;
import net.wanji.opt.bo.SceneMetricsDetailBO;
import net.wanji.opt.service.impl.SceneEvaluateServiceImpl;
import net.wanji.opt.vo.*;
import net.wanji.opt.vo.SceneMetricsDetailVO;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.util.List;

/**
 * 场景评价
 *
 * @author Kent HAN
 */
@Api(value = "SceneEvaluateController", description = "场景评价")
@RequestMapping("/sceneEvaluate")
@RestController
public class SceneEvaluateController {
    @Value("${mock.flag:false}")
    private boolean isMock = false;
    private final SceneEvaluateServiceImpl sceneEvaluateService;

    public SceneEvaluateController(SceneEvaluateServiceImpl sceneEvaluateService) {
        this.sceneEvaluateService = sceneEvaluateService;
    }


    @ApiOperation(value = "异常事件时空分布", notes = "异常事件时空分布", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/abnormalDistribute",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SceneEvaluateAbnormalDistributeVO.class),
    })
    public JsonViewObject abnormalDistribute(@RequestBody CrossIdAndStartEndDateBO bo) {
        if(isMock == true){
//            bo.setCrossId("12Q5V099C30");
//            bo.setStartDate(DateUtil.parse("2025-04-07 08:50:00", "yyyy-MM-dd HH:mm:ss"));
//            bo.setEndDate(DateUtil.parse("2025-04-07 16:30:00", "yyyy-MM-dd HH:mm:ss"));
        }
        SceneEvaluateAbnormalDistributeVO res = sceneEvaluateService.abnormalDistribute(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "异常事件详情", notes = "异常事件详情", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/abnormalDetail",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SceneEvaluateAbnormalDetailVO.class),
    })
    public JsonViewObject abnormalDetail(@RequestBody AbnormalDetailBO abnormalDetailBO) throws ParseException {
        SceneEvaluateAbnormalDetailVO res = sceneEvaluateService.abnormalDetail(abnormalDetailBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "曲线图", notes = "曲线图", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/metricsDetail",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SceneMetricsDetailVO.class),
    })
    public JsonViewObject schemeProblems(@RequestBody SceneMetricsDetailBO bo)
            throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
        List<SceneMetricsDetailVO> res = sceneEvaluateService.metricsDetail(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "转向级指标", notes = "转向级指标", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/metricsTurn",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SceneEvaluateMetricsTurnVO.class),
    })
    public JsonViewObject metricsTurn(@RequestBody MetricsTurnAndLaneBO bo) throws ParseException {
        SceneEvaluateMetricsTurnVO res = sceneEvaluateService.metricsTurn(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "车道级指标", notes = "车道级指标", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/metricsLane",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SceneEvaluateMetricsLaneVO.class),
    })
    public JsonViewObject metricsLane(@RequestBody MetricsTurnAndLaneBO bo) throws ParseException {
        SceneEvaluateMetricsLaneVO res = sceneEvaluateService.metricsLane(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "场景评价热力图", notes = "场景评价热力图", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/heatMap",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SceneEvaluateHeatMapVO.class),
    })
    public JsonViewObject heatMap(@RequestBody HeatMapBO bo) {
        List<SceneEvaluateHeatMapVO> res = sceneEvaluateService.heatMap(bo);
        return JsonViewObject.newInstance().success(res);
    }
}